/*
 * [TestCSVReader.java]
 *
 * Summary: Test CSVReader to read a csv file.
 *
 * Copyright: (c) 2002-2011 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  1.0 2010-12-31 initial version
 */
package com.mindprod.csv;

import java.io.BufferedReader;
import java.io.EOFException;
import java.io.FileReader;
import java.io.IOException;

import static java.lang.System.err;
import static java.lang.System.out;

/**
 * Test CSVReader to read a csv file.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 1.0 2010-12-31 initial version
 * @since 2010-12-31
 */
public final class TestCSVReader
    {
    // --------------------------- main() method ---------------------------

    /**
     * Test driver
     *
     * @param args not used
     *
     * @noinspection InfiniteLoopStatement
     */
    public static void main( String[] args )
        {
        try
            {
            // read test file, and dump it out.
            // reader, separatorChar, quoteChar, commentChars, hideComments, trimQuoted, trimUnquoted allowMultipleLineFields
            CSVReader csv = new CSVReader( new BufferedReader( new FileReader( "test.csv" ) ),
                    ',',  /* separatorChar between fields */
                    '\"', /* quoteChar to surround fields containing commas */
                    "#",  /* char that starts comments */
                    false /* hide comments */,
                    true /* trimQuoted */,
                    true /* trimUnquoted */,
                    true /* allow multiline fields multiline */
            );
            try
                {
                while ( true )
                    {
                    final String s = csv.get();
                    // we could also use more specific get methods.
                    // csv.getAllFieldsInLine();
                    // csv.skip();
                    // csv.skipToNextLine();
                    // csv.getBoolean();
                    // csv.getDouble()
                    // csv.getFloat()
                    // csv.getInt()
                    // csv.getLong()
                    // csv.getYYYYMMDD()
                    // csv.getLineCount()
                    // csv.wasComment()

                    if ( s == null )
                        {
                        out.println( "(end of line)" );
                        }
                    else if ( csv.wasComment() )
                        {
                        out.println( " <" + s + ">" );
                        }
                    else
                        {
                        out.println( "  [" + s + "]" );
                        }
                    // we do not demonstrate .skipToNextLine and .readAllFieldsInLine here.
                    }
                }
            catch ( EOFException e )
                {
                // normal termination.
                csv.close();
                }
            }
        catch ( IOException e )
            {
            err.println();
            e.printStackTrace( err );
            err.println();
            }
        }
    }
