/*
 * [PrepareUnicodeRanges.java]
 *
 * Summary: Prepare table about Unicode 6 ranges.
 *
 * Copyright: (c) 2011 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  1.0 2011-02-27 initial version
 */
package com.mindprod.unicode;

import com.mindprod.common15.STA;
import com.mindprod.csv.CSVReader;
import com.mindprod.fastcat.FastCat;
import com.mindprod.http.Get;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.EOFException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.URL;

import static java.lang.System.out;

/**
 * Prepare table about Unicode 6 ranges.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 1.0 2011-02-27 initial version
 * @since 2011-02-27
 */
public class PrepareUnicodeRanges
    {
    // --------------------------- main() method ---------------------------

    /**
     * main method
     *
     * @param args not used
     *
     * @throws java.io.IOException if IO trouble
     */
    public static void main( final String[] args ) throws IOException
        {
        // O P E N
        final CSVReader r = new CSVReader( new BufferedReader( new FileReader( "E:/com/mindprod/unicode/unicoderanges.csv" ) ) );
        final FileOutputStream fos = new FileOutputStream( "E:/mindprod/jgloss/include/unicoderanges.html", false /* append */ );
        final BufferedOutputStream bos = new BufferedOutputStream( fos, 16384 /* 16K bytes, 25% of allocation is optimal */ );
        final OutputStreamWriter eosw = new OutputStreamWriter( bos, "UTF-8" );
        final BufferedWriter bw = new BufferedWriter( eosw, 24576 /* 24K chars (48K bytes), 75% of allocation is optimal */ );
        final PrintWriter w = new PrintWriter( bw, false /* auto flush on println */ );
        try
            {
            while ( true )
                {
                final int startOfRange = r.getHexInt();
                int version = r.getInt();
                int size = r.getInt();
                final String sampleGlyph = r.get();
                final String use = r.get();
                final String desc = r.get();
                if ( desc != null )
                    {
                    r.skipToNextLine();
                    }

                final String displayHex = ( startOfRange > 0xffff ? STA.toLZHexString( startOfRange, 8 ) : STA.toLZHexString( startOfRange, 4 ) ).toUpperCase();
                final String pdf;
                if ( startOfRange <= 0xffff )
                    {
                    pdf = 'U' + STA.toLZHexString( startOfRange, 4 ).toUpperCase() + ".pdf";
                    }
                else if ( startOfRange <= 0xfffff )
                    {
                    pdf = 'U' + STA.toLZHexString( startOfRange, 5 ).toUpperCase() + ".pdf";
                    }
                else if ( startOfRange <= 0xffffff )
                    {
                    pdf = 'U' + STA.toLZHexString( startOfRange, 6 ).toUpperCase() + ".pdf";
                    }
                else if ( startOfRange <= 0xfffffff )
                    {
                    pdf = 'U' + STA.toLZHexString( startOfRange, 7 ).toUpperCase() + ".pdf";
                    }
                else
                    {
                    pdf = 'U' + STA.toLZHexString( startOfRange, 8 ).toUpperCase() + ".pdf";
                    }
                if ( size == 0 )
                    {
                    final Get p = new Get();
                    final String response = p.send( new URL( "http://www.unicode.org/charts/PDF/" + pdf ), "UTF-8" );
                    size = response.length();
                    out.println( displayHex + ", " + size );
                    }
                FastCat sb = new FastCat( 23 );

                // link via hex range
                sb.append( "<tr><td class=\"hex\">" );
                if ( version == 6 )
                    {
                    sb.append( "&#x2076;" );
                    }
                sb.append( "<a class=\"pdf\" href=\"http://www.unicode.org/charts/PDF/" );
                sb.append( pdf );
                sb.append( "\">" );
                sb.append( displayHex );
                sb.append( "</a></td>" );
                // size of pdf in k
                sb.append( "<td align=\"right\" class=\"unobtrusive\">" );
                sb.append( ( size + 1023 ) / 1024 );
                sb.append( "k</td>" );

                // sample glyph
                if ( sampleGlyph.endsWith( ".png" ) )
                    {  // in form of a png image. sampleGlyph
                    sb.append( "<td align=\"center\"><!-- macro Image unicode/" );
                    sb.append( sampleGlyph );
                    sb.append( " {sample glyph from range} --></td>" );
                    }
                else if ( sampleGlyph.length() > 0 )
                    {
                    // in form of an entity.
                    sb.append( "<td class=\"behold\">" );
                    sb.append( sampleGlyph );
                    sb.append( "</td>" );
                    }
                else
                    {
                    // no glyph.
                    sb.append( "<td></td>" );
                    }

                // use:desc
                sb.append( "<td><a class=\"pdf\" href=\"http://www.unicode.org/charts/PDF/" );
                sb.append( pdf );
                sb.append( "\">" );
                sb.append( use );
                sb.append( "</a>" );
                if ( desc != null )
                    {
                    sb.append( ": " );
                    sb.append( desc );
                    }
                sb.append( "</td></tr>" );
                w.println( sb.toString() );
                }
            }
        catch ( EOFException e )
            {
            r.close();
            w.close();
            }
        }
    }
