/*
 * [Unicode.java]
 *
 * Summary: Java applet to display all of the characters in the Unicode character set.
 *
 * Copyright: (c) 1996-2011 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  1.0 1996-01-01 initial version
 *  1.1 1996-01-01 convert to CMP standards.
 *  1.2 2000-01-01 add Javadoc, fix deprecated methods, ANT build, Lucida Sans
 *                 font, used ActionListener to avoid action method.
 *  1.3 2000-01-01 change font names to new undeprecated names.
 *  1.4 2000-01-01 convert to JDK 1.1+ event model, using listeners and
 *                 anonymous inner classes.
 *  1.5 2006-03-05 reformat with IntelliJ, add Javadoc
 *  1.6 2008-01-02 add pad and icons
 *  1.7 2010-01-07 add title, about box and instructions
 *  1.8 2010-11-27 convert to Swing and allow any Font. Now requires JDK 1.5+.
 */
package com.mindprod.unicode;
/*
Copyright: (c) 1996-2011 Nic Fulton, nic.fulton@reuters.com, Reuters Ltd, http://www.reuters.com. Roedy Green, Canadian Mind Products (polishing)
All Rights Reserved. Permission to use, copy,
modify, and distribute this software and its documentation for NON-COMMERCIAL
or COMMERCIAL purposes and without fee is hereby granted. REUTERS MAKES NO
REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
REUTERS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT
OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES. THIS
SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE CONTROL
EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE PERFORMANCE, SUCH AS
IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT NAVIGATION OR COMMUNICATION
SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE SUPPORT MACHINES, OR WEAPONS
SYSTEMS, IN WHICH THE FAILURE OF THE SOFTWARE COULD LEAD DIRECTLY TO DEATH,
PERSONAL INJURY, OR SEVERE PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK
ACTIVITIES"). REUTERS SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY
OF FITNESS FOR HIGH RISK ACTIVITIES. This cop-out clause is a copy of that
from JavaSoft.
     */
/**
 * Java applet to display all of the characters in the Unicode character set.
 * <p/>
 * The applet displays all 256 characters in a unicode byte block, with an
 * offset specified by two 4 bit values (i.e. one full byte).
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 1.8 2010-11-27 convert to Swing and allow any Font. Now requires JDK 1.5+.
 * @since 1996-01-01
 */

import com.mindprod.common11.Build;
import com.mindprod.common11.CMPAboutBox;
import com.mindprod.common11.FontFactory;
import com.mindprod.common11.VersionCheck;
import com.mindprod.common13.HybridJ;
import com.mindprod.common15.Laf;

import javax.swing.JApplet;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import java.awt.Color;
import java.awt.Container;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.util.Arrays;
import java.util.Vector;

/**
 * Displays complete Unicode 16 bit glyphs in any supported font.
 */
public final class Unicode extends JApplet
    {
    // ------------------------------ CONSTANTS ------------------------------

    /**
     * height of applet box in pixels. Does not include surrounding frame.
     */
    private static final int APPLET_HEIGHT = 775;

    /**
     * Width of applet box in pixels.
     */
    private static final int APPLET_WIDTH = 560;

    /**
     * starting font
     */
    private static final String defaultFontFamily = "Dialog";

    /**
     * undisplayed copyright notice
     */
    public static final String EMBEDDED_COPYRIGHT =
            "Copyright: (c) 1996-2011 Nic Fulton nic.fulton@reuters.com, 1996-2010 Roedy Green, Canadian Mind Products http://mindprod.com";

    private static final String RELEASE_DATE = "2010-11-27";

    private static final String TITLE_STRING = "Unicode Font Viewer";

    private static final String VERSION_STRING = "1.8";

    private static final Color BACKGROUND_FOR_APPLET = new Color( 0xfaf0e6 )/* linen */;

    private static final Color BACKGROUND_FOR_INSTRUCTIONS = Color.white;

    private static final Color FOREGROUND_FOR_INSTRUCTIONS = new Color( 0x006400 )/* dark green */;

    /**
     * for titles
     */
    private static final Color FOREGROUND_FOR_TITLE = new Color( 0xdc143c );

    /**
     * for for titles
     */
    private static final Font FONT_FOR_TITLE = FontFactory.build( "Dialog", Font.BOLD, 16 );

    /**
     * for for title second line
     */
    private static final Font FONT_FOR_TITLE2 = FontFactory.build( "Dialog", Font.PLAIN, 14 );

    // ------------------------------ FIELDS ------------------------------

    private Container contentPane;

    /**
     * choice of all supported fonts
     */
    private JComboBox fontChoices;

    private JLabel title;

    private JLabel title2;

    /**
     * how to use the program instructions
     */
    private JTextArea instructions;

    /**
     * grd bottom and right side bars
     */
    private RigidPanel theRigidPanel;

    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------

    /**
     * default constructor for Unicode JApplet
     */
    public Unicode()
        {
        }

    /**
     * Called by the browser or applet viewer to inform
     * this applet that it is being reclaimed and that it should destroy
     * any resources that it has allocated.
     *
     * @override
     */
    public void destroy()
        {
        contentPane = null;
        fontChoices = null;
        instructions = null;
        theRigidPanel = null;
        title2 = null;
        title = null;
        }

    /**
     * Called by the browser or applet viewer to inform
     * this applet that it has been loaded into the system.
     *
     * @override
     */
    @Override
    public void init()
        {
        contentPane = this.getContentPane();
        if ( !VersionCheck.isJavaVersionOK( 1, 5, 0, contentPane ) )
            {
            return;
            }
        buildMenu(); // also initial L&F
        contentPane.setLayout( new GridBagLayout() );
        contentPane.setBackground( BACKGROUND_FOR_APPLET );
        buildComponents();
        layoutComponents();
        this.validate();// calculate the layouts
        this.setVisible( true );// Lights, Camera, Action.
        }

    // -------------------------- OTHER METHODS --------------------------

    /**
     * create all components for Applets.
     */
    private void buildComponents()
        {
        title = new JLabel( TITLE_STRING + " " + VERSION_STRING );
        title.setFont( FONT_FOR_TITLE );
        title.setForeground( FOREGROUND_FOR_TITLE );
        title2 = new JLabel(
                "released:" +
                RELEASE_DATE +
                " build:" +
                Build.BUILD_NUMBER );
        title2.setFont( FONT_FOR_TITLE2 );
        title2.setForeground( FOREGROUND_FOR_TITLE );
        theRigidPanel = new RigidPanel();
        buildFontChoices();
        fontChoices.addActionListener( new ActionListener()
        {
        public void actionPerformed( ActionEvent e )
            {
            // pass String font family name not Font object
            theRigidPanel.setFontFamily( ( String ) fontChoices.getSelectedItem() );
            }
        } );
        instructions = new JTextArea( "To view a given hex code:\n" +
                                      "Select the first digit with the corresponding white square at the bottom.\n" +
                                      "Then select the second digit with the corresponding white square on the right.\n" +
                                      "Then sight (but don't click) the third digit with the corresponding black digit on the bottom.\n" +
                                      "Then sight (but don't click) the fourth digit with the corresponding black square on the right.\n" +
                                      "Select various fonts to see how the glyphs change.\n" +
                                      "You can also click a grid square to see it magnified in the bottom right corner.",
                7 /* rows */, 80 /* cols */ );
        instructions.setBackground( BACKGROUND_FOR_INSTRUCTIONS );
        instructions.setForeground( FOREGROUND_FOR_INSTRUCTIONS );
        instructions.setFont( FontFactory.build( "Dialog", Font.PLAIN, 12 ) );
        instructions.setEditable( false );
        }

    /**
     * set up fontChoices Widget with list of all possible fonts, but not listeners.
     */
    private void buildFontChoices()
        {
        // font names
        final GraphicsEnvironment ge = GraphicsEnvironment
                .getLocalGraphicsEnvironment();
        final String[] fontNames = ge.getAvailableFontFamilyNames();
        // use Vector instead of ArrayList because JComboBox wants a vector
        final Vector<String> v = new Vector<String>( fontNames.length + 1 );
        v.addAll( Arrays.asList( fontNames ) );
        fontChoices = new JComboBox( v );
        fontChoices.setMaximumSize( fontChoices.getPreferredSize() );
        fontChoices.setEditable( true );
        fontChoices.setSelectedItem( defaultFontFamily );
        }

    /**
     * build a menu with Look & Feel and About across the top
     */
    private void buildMenu()
        {
        final JMenuBar menubar = new JMenuBar();
        setJMenuBar( menubar );
        menubar.add( Laf.buildLookAndFeelMenu() );
        final JMenu menuHelp = new JMenu( "Help" );
        menubar.add( menuHelp );
        final JMenuItem aboutItem = new JMenuItem( "About" );
        menuHelp.add( aboutItem );
        aboutItem.addActionListener( new ActionListener()
        {
        public void actionPerformed( ActionEvent e )
            {
            // open about frame
            new CMPAboutBox( TITLE_STRING,
                    VERSION_STRING,
                    "Display Unicode Characters",
                    "in range 0..FFFF",
                    "freeware",
                    RELEASE_DATE,
                    1996 /* first copyrighted */,
                    "Roedy Green",
                    "UNICODE",
                    "1.1" );
            }
        } );
        }

    private void layoutComponents()
        {
        /*  0---------1------2
            0  titles1 title2 fontChoice
            1  rigid panel
            2  instructions
        */
        // x y w h wtx wty anchor fill T L B R padx pady
        contentPane.add( title,
                new GridBagConstraints( 0,
                        0,
                        1,
                        1,
                        0.0,
                        0.0,
                        GridBagConstraints.WEST,
                        GridBagConstraints.NONE,
                        new Insets( 10, 10, 10, 5 ),
                        0,
                        0 ) );
        // x y w h wtx wty anchor fill T L B R padx pady
        contentPane.add( title2,
                new GridBagConstraints( 1,
                        0,
                        1,
                        1,
                        0.0,
                        0.0,
                        GridBagConstraints.WEST,
                        GridBagConstraints.NONE,
                        new Insets( 10, 5, 10, 5 ),
                        0,
                        0 ) );
        contentPane.add( fontChoices,
                new GridBagConstraints( 2,
                        0,
                        1,
                        1,
                        0.0,
                        0.0,
                        GridBagConstraints.CENTER,
                        GridBagConstraints.BOTH,
                        new Insets( 0, 10, 5, 5 ),
                        0,
                        0 ) );
        // x y w h wtx wty anchor fill T L B R padx pady
        contentPane.add( theRigidPanel,
                new GridBagConstraints( 0,
                        1,
                        3,
                        1,
                        10.0,
                        10.0,
                        GridBagConstraints.CENTER,
                        GridBagConstraints.BOTH,
                        new Insets( 5, 10, 0, 10 ),
                        0,
                        0 ) );
        // x y w h wtx wty anchor fill T L B R padx pady
        contentPane.add( instructions,
                new GridBagConstraints( 0,
                        2,
                        3,
                        1,
                        0.0,
                        1.0,
                        GridBagConstraints.CENTER,
                        GridBagConstraints.BOTH,
                        new Insets( 5, 10, 10, 10 ),
                        0,
                        0 ) );
        }

    // -------------------------- INNER CLASSES --------------------------

    /*
     creates the whole grind with right and top panels.
     */
    static final class RigidPanel extends JPanel
        {
        // ------------------------------ FIELDS ------------------------------

        private final TextPanel theTextPanel;

        // -------------------------- PUBLIC INSTANCE  METHODS --------------------------

        /**
         * Set the font to the selected font.
         *
         * @param theFontFamily to draw the chars in
         */
        public void setFontFamily( String theFontFamily )
            {
            theTextPanel.setFontFamily( theFontFamily );
            }

        // --------------------------- CONSTRUCTORS ---------------------------

        /**
         * constructor
         */
        RigidPanel()
            {
            // set the layout to null. i.e. hard code in all the positions
            // so that they don't go 'drifting' with unexpected layout
            // manager behaviour
            setLayout( null );
            setBackground( Color.white );
            this.setSize( 475, 475 );
            // A new object to contain the single 'zoomed' character
            final CharPanel theCharPanel = new CharPanel();
            add( theCharPanel );
            theCharPanel.setBounds( 400, 400, 75, 75 );
            // A new object to display the grid of characters
            theTextPanel = new TextPanel( theCharPanel );
            add( theTextPanel );
            theTextPanel.setBounds( 0, 0, 400, 400 );
            // A rather crude slider for the base
            final BottomPanel theBottomPanel = new BottomPanel( theTextPanel );
            add( theBottomPanel );
            theBottomPanel.setBounds( 0, 400, 400, 75 );
            // A rather crude slider for the side.
            // On reflection maybe I should have had both the bottom
            // and right sliders in the same class with an orientation...
            final RightPanel theRightPanel = new RightPanel( theTextPanel );
            add( theRightPanel );
            theRightPanel.setBounds( 400, 0, 75, 400 );
            }
        }

    // --------------------------- main() method ---------------------------

    /**
     * Allow this applet to run as as application as well.
     *
     * @param args command line arguments ignored.
     */
    public static void main( String args[] )
        {
        HybridJ.fireup( new Unicode(),
                TITLE_STRING + " " + VERSION_STRING,
                APPLET_WIDTH,
                APPLET_HEIGHT );
        }// end main;
    }

/**
 * The class that contains the grid of characters.
 */
final class TextPanel extends JPanel
    {
    // ------------------------------ FIELDS ------------------------------

    /**
     * panel that contains grid of rendered chars
     */
    private final CharPanel theCharPanel;

    /**
     * current font to use for rendering
     */
    private String theFontFamily = "Monospaced";

    /**
     * row group 0..15
     */
    private int colGroup = 0;

    /**
     * col group 0..15
     */
    private int rowGroup = 0;

    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------

    /**
     * constructor for text panel
     *
     * @param theCharPanel panel that contains grid of rendered chars
     */
    public TextPanel( final CharPanel theCharPanel )
        {
        this.theCharPanel = theCharPanel;
        this.addMouseListener( new MouseAdapter()
        {
        /**
         * Invoked when a mouse button has been pressed on a component.
         */
        public void mousePressed( MouseEvent e )
            {
            int i = e.getX() / 25;
            int j = e.getY() / 25;
            theCharPanel.setChar( j, i, rowGroup, colGroup );
            }
        } );
        }

    /**
     * On paint, draw all the characters in their little boxes.
     */
    public void paintComponent( Graphics g )
        {
        super.paintComponent( g );
        // Set the font to the selected font.
        g.setFont( new Font( theFontFamily, Font.PLAIN, 16 ) );
        g.drawRect( 0, 0, 400 - 1, 400 - 1 );
        // draw the grid
        for ( int i = 1; i < 16; i++ )
            {
            g.drawLine( 0, i * 25, 400, i * 25 );
            g.drawLine( i * 25, 0, i * 25, 400 );
            }
        // dummy array to contain the character being drawn
        // There is no Graphics.drawChar() method. Why not, surely
        // Graphics.drawString() makes the drawChars unnecessary.
        char Chars[] = new char[ 1 ];
        // draw all the characters
        for ( int i = 0; i < 16; i++ )
            {
            for ( int j = 0; j < 16; j++ )
                {
                // Calculate the characters UNICODE value.
                int anInt = j + i * 16 + rowGroup * 256 + colGroup * 4096;
                Chars[ 0 ] = ( char ) anInt;
                // See comment above
                g.drawChars( Chars, 0, 1, i * 25 + 4, j * 25 + 21 );
                }
            }
        }

    /**
     * Set the offset value of the 256 character window when the sliders are being used.
     *
     * @param i column group 0..15 of char to display
     */
    public void setColGroup( int i )
        {
        colGroup = i;
        repaint();
        }

    /**
     * Set the font to the selected font.
     *
     * @param theFontFamily to draw the chars in
     */
    public void setFontFamily( String theFontFamily )
        {
        this.theFontFamily = theFontFamily;
        theCharPanel.setFontFamily( theFontFamily );
        repaint();
        }

    /**
     * Set the offset value of the 256 character window when the sliders are being used.
     *
     * @param j row group of char to display
     */
    public void setRowGroup( int j )
        {
        rowGroup = j;
        repaint();
        }
    }

/**
 * The little box with the character in. this also displays the HEX value of the character that you click on
 */
final class CharPanel extends JPanel
    {
    // ------------------------------ FIELDS ------------------------------

    /**
     * current font we are using for rendering
     */
    private String theFontFamily = "Monospaced";

    /**
     * current character we are rendering
     */
    private char theChar;

    /**
     * first digit of unicode xxxx 0..16
     */
    private int digit1 = 0;

    /**
     * second digit of unicode xxxx 0..16
     */
    private int digit2 = 0;

    /**
     * third digit of unicode xxxx 0..16
     */
    private int digit3 = 0;

    /**
     * fourth digit of unicode xxxx 0..16
     */
    private int digit4 = 0;

    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------

    /**
     * constructor
     */
    public CharPanel()
        {
        theChar = ( char ) 0x0020;
        // no listeners
        }

    /**
     * Oh, argghh, what horrible code, but it works.
     */
    public void paintComponent( Graphics g )
        {
        super.paintComponent( g );
        String aString =
                Integer.toString( digit1, 16 )
                + Integer.toString( digit2, 16 )
                + Integer.toString( digit3, 16 )
                + Integer.toString( digit4, 16 );
        // x,y is bottom left corner of text
        g.drawString( aString, 28, 22 );
        g.drawRect( 25, 25, 49, 49 );
        g.drawRect( 26, 26, 47, 47 );
        // Get the current font then make a new BIG one (triple the size).
        Font fontMed = g.getFont();
        // Don't use Font Factory. Want precise font, not best
        Font fontBig = new Font( theFontFamily, fontMed.getStyle(), fontMed.getSize() * 3 );
        g.setFont( fontBig );
        char Chars[] = new char[ 1 ];
        Chars[ 0 ] = theChar;
        g.drawChars( Chars, 0, 1, 35, 66 );
        }

    /**
     * On a click change the characters value. *
     *
     * @param a first hex digit
     * @param b second hex digit
     * @param c third hex digit
     * @param d fourth hex digit
     */
    public void setChar( int a, int b, int c, int d )
        {
        digit1 = d;
        digit2 = c;
        digit3 = b;
        digit4 = a;
        int i = digit4 + digit3 * 16 + digit2 * 256 + digit1 * 4096;
        theChar = ( char ) i;
        repaint();
        }

    /**
     * Set the font to the selected one
     *
     * @param theFont font to draw the chars
     */
    public void setFontFamily( String theFont )
        {
        this.theFontFamily = theFont;
        repaint();
        }
    }

/**
 * The rather crude slider. This one is for the bottom of the applet.
 */
final class BottomPanel extends JPanel
    {
    // ------------------------------ FIELDS ------------------------------

    /**
     * column group 0..15 in bottom panel
     */
    private int colGroup = 0;

    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------

    /**
     * constructor constructor panel to contain pieces
     *
     * @param theTextPanel display the controls to select the column group
     */
    public BottomPanel( final TextPanel theTextPanel )
        {
        this.addMouseListener( new MouseAdapter()
        {
        /**
         * Invoked when a mouse button has been pressed on a component.
         * Catch a mousedown and reset the position of the slider if it has
         * changed. Redraw the character grid.
         */
        public void mousePressed( MouseEvent e )
            {
            int i = e.getX() / 25;
            if ( colGroup != i && i < 16 && i >= 0 )
                {
                colGroup = i;
                repaint();
                theTextPanel.setColGroup( i );
                }
            }
        } );
        this.addMouseMotionListener( new MouseMotionAdapter()
        {
        /**
         * catch a mouse drag and move the slider if the position has
         * changed. Redraw the character grid
         */
        public void mouseDragged( MouseEvent e )
            {
            int i = e.getX() / 25;
            if ( colGroup != i && i < 16 && i >= 0 )
                {
                colGroup = i;
                repaint();
                theTextPanel.setColGroup( i );
                }
            }
        } );
        }

    /**
     * The paint method. This also draws the axes.
     */
    public void paintComponent( Graphics g )
        {
        super.paintComponent( g );
        g.fillRect( 0, 0, 400, 25 );
        Color aColor = g.getColor();
        g.setColor( Color.white );
        for ( int i = 0; i < 16; i++ )
            {
            g.drawString( Integer.toString( i, 16 ), i * 25 + 7, 20 );
            }
        g.setColor( aColor );
        g.drawRect( 0, 50, 400 - 1, 24 );
        for ( int i = 1; i < 16; i++ )
            {
            g.drawLine( i * 25, 50, i * 25, 75 );
            }
        g.fillRect( colGroup * 25, 50, 25, 25 );
        }
    }

/**
 * The right slider, well sort of, control extending Canvas
 */
final class RightPanel extends JPanel
    {
    // ------------------------------ FIELDS ------------------------------

    /**
     * row group 9..15
     */
    private int rowGroup = 0;

    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------

    /**
     * constructor right panel
     *
     * @param theTextPanel panel where we render the controls for row group
     */
    public RightPanel( final TextPanel theTextPanel )
        {
        this.addMouseListener( new MouseAdapter()
        {
        /**
         * Invoked when a mouse button has been pressed on a component.
         * catch a mousedown and reset the position of the slider if it has
         * changed. Redraw the character grid.
         */
        public void mousePressed( MouseEvent e )
            {
            int j = e.getY() / 25;
            if ( rowGroup != j && j < 16 && j >= 0 )
                {
                rowGroup = j;
                repaint();
                theTextPanel.setRowGroup( j );
                }
            }
        } );
        this.addMouseMotionListener( new MouseMotionAdapter()
        {
        /**
         * catch a mouse drag and move the slider if the position has
         * changed. Redraw the character grid
         */
        public void mouseDragged( MouseEvent e )
            {
            int j = e.getY() / 25;
            if ( rowGroup != j && j < 16 && j >= 0 )
                {
                rowGroup = j;
                repaint();
                theTextPanel.setRowGroup( j );
                }
            }
        } );
        }

    public void paintComponent( Graphics g )
        {
        super.paintComponent( g );
        g.fillRect( 0, 0, 25, 400 );
        Color aColor = g.getColor();
        g.setColor( Color.white );
        for ( int i = 0; i < 16; i++ )
            {
            g.drawString( Integer.toString( i, 16 ), 7, i * 25 + 20 );
            }
        g.setColor( aColor );
        g.drawRect( 50, 0, 24, 400 - 1 );
        for ( int i = 1; i < 16; i++ )
            {
            g.drawLine( 50, i * 25, 75, i * 25 );
            }
        g.fillRect( 50, rowGroup * 25, 25, 25 );
        }
    }
